@@ -506,8 +506,9 @@ cond_indirect <- function(x,
               }
           } else {
             mc_out <- do_mc(fit = fit,
-                                R = R,
-                                seed = seed)
+                            R = R,
+                            seed = seed,
+                            progress = progress)
           }
       }
     if (boot_ci) {
@@ -610,6 +611,7 @@ cond_indirect <- function(x,
                                      format = "f"), "%")
         out0$mc_ci <- boot_ci1
         out0$level <- level
+        out0$mc_se <- stats::sd(out0$mc_indirect, na.rm = TRUE)
         if (save_mc_out) {
             out0$mc_out <- mc_out
           } else {
@@ -646,6 +648,7 @@ cond_indirect <- function(x,
         out0$boot_ci <- boot_ci1
         out0$level <- level
         out0$boot_p <- est2p(out0$boot_indirect)
+        out0$boot_se <- stats::sd(out0$boot_indirect, na.rm = TRUE)
         if (save_boot_out) {
             out0$boot_out <- boot_out
           } else {
